
local EMPOWERING_SPELL_ID = 227907

local empoweringSpellName
local tooltipScanner
local tooltipName = "BagArtifactPowerTooltipScanner"
local totalPower;
local f  = CreateFrame("frame",nil,UIParent);
local numberOverlay = {};
local ldb =  LibStub:GetLibrary("LibDataBroker-1.1");
local ldbPower;

function f:OnLoad()
	empoweringSpellName = GetSpellInfo(EMPOWERING_SPELL_ID)

	f:SetScript("OnEvent",f.EventHandler);	
	f:RegisterEvent("BAG_UPDATE_DELAYED");
	hooksecurefunc("ContainerFrame_OnHide", f.ScanContainers);
	hooksecurefunc("ContainerFrame_OnShow", f.ScanContainers);
	tooltipScanner = CreateFrame("GameTooltip", tooltipName, nil, "GameTooltipTemplate")
	ldbPower = ldb:NewDataObject("Bag Artifact Power", { type = "data source", label ="", text = "", icon = "Interface\\ICONS\\INV_Artifact_XP05"});
end

function f:RoundNumber(val, decimal)
  if decimal then
    return math.floor( (val * 10^decimal) + 0.5) / (10^decimal)
  else
    return math.floor(val + 0.5)
  end
end

function f:FormatNumber(n)
    if n >= 1000000 then
        return f:RoundNumber(n / 1000000, 2) .. "M"
    elseif n >= 100000 then
        return f:RoundNumber(n / 1000) .. "K"
    elseif n >= 10000 then
        return f:RoundNumber(n / 1000, 1) .. "K"
    else
        return n .. ""
    end
end

function f:CreateNumberOverlay(bag,slot,power)
	local currentOverlay;		
	if not(numberOverlay[bag..'-'..slot]) then	
		currentOverlay = f:CreateFontString(nil, "OVERLAY", "GameFontNormalLarge");		
		numberOverlay[bag..'-'..slot] = currentOverlay;
	else	
		currentOverlay = numberOverlay[bag..'-'..slot];
	end
	currentOverlay.active = true;
	currentOverlay:SetText(power);	
end

function f:HideOverlays(deactivate)
	local v;
	for _,v in pairs(numberOverlay) do
		v:Hide();
		if(deactivate==true) then
			v.active = false;
		end		
	end
end

function f:ScanContainers()
	f:HideOverlays(false);
    local i,j,bag,slot
    for i = 1, NUM_CONTAINER_FRAMES do
        for j = 1, MAX_CONTAINER_ITEMS do
			local containerFrame = _G["ContainerFrame"..i.."Item"..j]
			if(containerFrame:IsVisible()) then
				bag, slot = containerFrame:GetParent():GetID(), containerFrame:GetID()
				
				if(numberOverlay[bag..'-'..slot] and numberOverlay[bag..'-'..slot].active == true) then
					
					numberOverlay[bag..'-'..slot]:SetParent(containerFrame)
					numberOverlay[bag..'-'..slot]:SetPoint("CENTER", 0, 0)
					numberOverlay[bag..'-'..slot]:Show(true);
					
				end
			end
        end
    end
end

function f:CheckBags()	
	f:HideOverlays(true);
	totalPower = 0;
	local itemLink,powerGain,rawPowerGain;
	for bag = 0, 4 do	
		for slot = 1, GetContainerNumSlots(bag) do
			itemLink = GetContainerItemLink(bag, slot)
			powerGain,rawPowerGain = f:GetItemLinkArtifactPower(itemLink);
			if powerGain then
				totalPower = totalPower + rawPowerGain	
				f:CreateNumberOverlay(bag,slot,powerGain)
			end
		end
	end
	f:ScanContainers()
	ldbPower.text = totalPower;
end

function f:EventHandler(event,...)
	if(event=="BAG_UPDATE_DELAYED") then
		f:CheckBags();	
	end
end

function f:GetItemLinkArtifactPower(itemLink)
    if itemLink then
        local itemSpell = GetItemSpell(itemLink)
        if itemSpell and itemSpell == empoweringSpellName then
            tooltipScanner:SetOwner(UIParent, "ANCHOR_NONE")
            tooltipScanner:SetHyperlink (itemLink)
			local tooltipText = _G[tooltipName.."TextLeft4"]:GetText()
			if(tooltipText == nil) then
				return nil
			end
			local rawap
            local ap = tooltipText:gsub("[,%.]", ""):match("%d.-%s")
            if ap then
                rawap = tonumber(ap)
                ap = f:FormatNumber(rawap)
            end
            tooltipScanner:Hide()
            return ap,rawap
        else
            return nil
        end
    else
        return nil
    end
end

f:OnLoad();
